# -*- coding: utf-8 -*-
'''
Created on 01.07.2010

@author: adm
'''
from artixds.objectdiscount import CheckObject, PositionObject, PositionObjectModifiers
from artixds.roster import RosterFactory
from datetime import datetime
import logging
log = logging.getLogger("CheckWrapper")

class CheckWrapper(object):
    """
    Contain check (as dictionary) in inner data
    """
    def __init__(self, check=None):
        """
        check - dictionary with check content
        """
        self.id = 0  # ревизия чека
        self.isModified = False  # был ли модифицирован чек?
        self.check = check
        self._nextPosNum = check.getNextPosNum() if check else None
        self.modifiedItems = {}  # модификации реальной позиции для получения списка позиции после структурных изменений
        self.createdPositions = {}  # список позиции после структурных изменений
        self.parents = {}
        self.checkDict = self.formCheckDict(check) if check else None
        self.dateTimeBeg = datetime.strptime(check.getDateTimeBeg().toString("yyyy-MM-ddThh:mm:ss"), "%Y-%m-%dT%H:%M:%S") if check else None
        self.positionsWithDiscount = []
        self.cwGoodsList = []
        self.positionsDiscount = set()  # примененные скидки
        self.positionsBonus = set()  # списанные бонусы
        self.positionsImpact = set()  # примененные воздествия
        # запомним позиции, к которым уже было применено воздействие
        if check:
            for impact in check.getDiscountList():
                discType = u'discountType' if u'discountType' in impact.totals else u'discounttype' if u'discounttype' in impact.totals else None
                if not discType:
                    continue

                for k in impact.details:
                    if impact.totals[discType] == 5:
                        self.positionsBonus.add(int(k))
                    else:
                        discountMode = u'discountMode' if u'discountMode' in impact.totals else u'discountmode'     # В тестах используется 'discountmode'
                        if impact.totals[discountMode] not in [6, 8]:   # Скидку на мелочь и скидку сертификатом, не считаем за скидку.
                            self.markPositionByDiscount(int(k))
                            self.positionsDiscount.add(int(k))
            # сохраняем список позиций чека, что бы не добывать его каждый раз
            goodsItemsKey = "goodsItems"  # старый дисконт
            if 'goodsitems' in self.checkDict:
                goodsItemsKey = "goodsitems"  # новый дисконт
            self.cwGoodsList = self.checkDict[goodsItemsKey]

        # необходимо сохранять прежние параметры при переходе внутрь условий
        self._basicNextPosNum = self._nextPosNum
        # модификации реальной позиции для получения списка позиции после структурных изменений
        self._basicModifiedItems = {}
        # список позиции после структурных изменений
        self._basicCreatedPositions = {}
        self._basicParents = {}
        self._basicPositionsWithDiscount = self.positionsWithDiscount[:]
        self._basicPositionsDiscount = self.positionsDiscount.copy ()
        self._basicPositionsBonus = self.positionsBonus.copy ()
        self._basicPositionsImpact = self.positionsImpact.copy ()

    def copy(self, checkWrapper):
        """
        Content copied to other object
        check - dictionary with check content
        """
        self.id = checkWrapper.id
        self.check = checkWrapper.check
        self.checkDict = checkWrapper.checkDict
        self._nextPosNum = checkWrapper._basicNextPosNum
        self.modifiedItems = checkWrapper._basicModifiedItems.copy()
        self.createdPositions = checkWrapper._basicCreatedPositions.copy()
        self.parents = checkWrapper._basicParents.copy()
        self.positionsWithDiscount = checkWrapper._basicPositionsWithDiscount[:]
        self.positionsDiscount = checkWrapper._basicPositionsDiscount.copy()
        self.positionsBonus = checkWrapper._basicPositionsBonus.copy()
        self.positionsImpact = checkWrapper._basicPositionsImpact.copy()
        self.cwGoodsList = checkWrapper.cwGoodsList
        self.dateTimeBeg = checkWrapper.dateTimeBeg
        return self

    def newGroup(self):
        log.debug("member modifiers for new group")
        self._basicNextPosNum = self._nextPosNum
        self._basicModifiedItems = self.modifiedItems.copy()
        self._basicCreatedPositions = self.createdPositions.copy()
        self._basicParents = self.parents.copy()
        self._basicPositionsWithDiscount = self.positionsWithDiscount[:]
        self._basicPositionsDiscount = self.positionsDiscount.copy()
        self._basicPositionsBonus = self.positionsBonus.copy()
        self._basicPositionsImpact = self.positionsImpact.copy()

    def markPositionByDiscount(self, posnum):
        log.debug("mark position with number %d" % posnum)
        self.positionsImpact.add(posnum)
        if not posnum in self.positionsWithDiscount:
            self.positionsWithDiscount.append(posnum)

    def getObjects(self):
        """
        Get all objects of check: CheckObject,PositionObject,GoodsKit
        """
        return self.createObjects()

    def getCheckObject(self):
        return CheckObject(self.checkDict)


    def createObjects(self):
        """
        Create all objects of check: CheckObject,PositionObject,GoodsKit
        """
        objects = []

        checkObj = CheckObject(self.checkDict)
        objects.append(checkObj)

        objects.extend(self.getGoodsList())

        goodsKit = self.createKit()
        objects.append(goodsKit)

        return objects

    def isFullMode(self):
        if "internaldiscountfullmode" in self.checkDict:
            return self.checkDict["internaldiscountfullmode"]

        return True

    def getGoodsList(self):
        objects = []
        for position in self.cwGoodsList:
            posnum = position["posnum"]
            if posnum in self.modifiedItems:
                for item in list(filter(lambda x: x.active, self.modifiedItems[posnum])):
                    posObject = PositionObject(position, item)
                    if posnum in self.positionsDiscount:
                        posObject.markAsDiscount()
                    if posnum in self.positionsBonus:
                        posObject.markAsBonus()
                    if item.posnum in self.positionsImpact:
                        posObject.markAsImpact()
                        posObject.markAsDiscountApplied()
                    objects.append(posObject)
            else:
                posObject = PositionObject(position)
                if posnum in self.positionsDiscount:
                    posObject.markAsDiscount()
                if posnum in self.positionsBonus:
                    posObject.markAsBonus()
                if posnum in self.positionsImpact:
                    posObject.markAsImpact()
                    posObject.markAsDiscountApplied()
                objects.append(posObject)
        return objects

    def createKit(self):
        return RosterFactory.getRosterByCheck(self)

    def getNextPositionNumber(self):
        """
        return next position number as max
        of all goodsitems and all stornoitems, plus 1
        """
        return self._nextPosNum

    def getDataTimeBeg(self):
        """
        return opening date of check
        """
        return self.dateTimeBeg

    def markAsPartOfKit(self, number):
        position = list(filter(lambda x: x.data["posnum"] == number, self.getGoodsList()))[0].data

        # может прийти изменение для уже измененной позиции нужно знать номер оригигальной
        # будем искать позицию среди созданных, если такой нет, то создадим
        posnum = position["posnum"]

        posroot = posnum
        while posroot in self.parents:
            posroot = self.parents[posroot]

        item = self.createdPositions[posnum] if posnum in self.createdPositions else None
        if not item:
            item = PositionObjectModifiers(posnum, True, position["cquant"], position["bquant"], position["sumi"],
                    position["sumn"], position["sumb"], position["sume"], position["packingmode"])
            self.modifiedItems[posroot] = [item]
            self.createdPositions[posnum] = item

        if posroot not in self.modifiedItems:
            self.modifiedItems[posroot] = []

        if posroot in self.modifiedItems and self.modifiedItems[posroot]:
            for item in self.modifiedItems[posroot]:
                if item.posnum == posnum:
                    item.partOfKit = True
        else:
            self.modifiedItems[posroot].append(item)

    def isPartOfKit(self, number):
        for rootPosnum in self.modifiedItems:
            for pos in self.modifiedItems[rootPosnum]:
                if pos.posnum is number:
                    return pos.partOfKit
        return False

    def branchPosition(self, number, branchDetails):
        position = list(filter(lambda x: x.data["posnum"] == number, self.getGoodsList()))[0].data

        # может прийти изменение для уже измененной позиции нужно знать номер оригигальной
        # будем искать позицию среди созданных, если такой нет, то создадим
        posnum = position["posnum"]

        posroot = posnum
        while posroot in self.parents:
            posroot = self.parents[posroot]

        item = self.createdPositions[posnum] if posnum in self.createdPositions else None
        if not item:
            item = PositionObjectModifiers(posnum, False, position["cquant"], position["bquant"], position["sumi"],
                    position["sumn"], position["sumb"], position["sume"], position["packingmode"])
            self.modifiedItems[posroot] = [item]
            self.createdPositions[posnum] = item
        item.active = False

        # для обнаруженной позиции сделаем бранч
        for detail in branchDetails:
            newItem = PositionObjectModifiers(detail.positionNumber, detail.partOfKit,
                    item.cquant * detail.quantity / item.bquant, detail.quantity,
                    item.sumi * detail.quantity / item.bquant,
                    item.sumn * detail.quantity / item.bquant,
                    item.sumb * detail.quantity / item.bquant,
                    item.sume * detail.quantity / item.bquant, item.packingMode)
            if posroot not in self.modifiedItems:
                self.modifiedItems[posroot] = []
            self.modifiedItems[posroot].append(newItem)
            self.createdPositions[detail.positionNumber] = newItem
            self.parents[detail.positionNumber] = posnum
            if detail.positionNumber >= self._nextPosNum:
                self._nextPosNum = detail.positionNumber + 1

    def unitePosition(self, number, qty, uniteDetails):
        '''
        Объединить позиции из uniteDetails в позицию с номером number
        qty - новое кол-во позиции с номером number
        '''
        position = list(filter(lambda x: x.data["posnum"] == number, self.getGoodsList()))[0].data

        posnum = position["posnum"]

        posroot = posnum
        while posroot in self.parents:
            posroot = self.parents[posroot]

        item = self.createdPositions[posnum] if posnum in self.createdPositions else None
        if not item:
            item = PositionObjectModifiers(posnum,  position["partOfKit"], position["cquant"], position["bquant"],
                    position["sumi"], position["sumn"], position["sumb"], position["sume"], position["packingmode"])
            self.modifiedItems[posroot] = [item]
            self.createdPositions[posnum] = item

        factor = qty / item.bquant
        item.cquant *= factor
        item.sumi *= factor
        item.sumn *= factor
        item.sumb *= factor
        item.sume *= factor
        item.bquant = qty

        # Удаляем куски из позиций в которых надо забрать
        for number, quantity in uniteDetails.items():
            position = list(filter(lambda x: x.data["posnum"] == number, self.getGoodsList()))[0].data
            posnum = position["posnum"]

            item = self.createdPositions[posnum] if posnum in self.createdPositions else None
            if not item:
                item = PositionObjectModifiers(posnum, position["partOfKit"], position["cquant"], position["bquant"],
                        position["sumi"], position["sumn"], position["sumb"], position["sume"], position["packingmode"])
                posroot = posnum
                while posroot in self.parents:
                    posroot = self.parents[posroot]
                self.modifiedItems[posroot] = [item]
                self.createdPositions[posnum] = item

            if item.bquant - quantity < 0.001:
                item.active = False
                continue

            factor = 1 - quantity / item.bquant
            item.cquant *= factor
            item.sumi *= factor
            item.sumn *= factor
            item.sumb *= factor
            item.sume *= factor
            item.bquant -= quantity

    def formCheckDict(self, check):
        result = check.toDict()
        # TODO: Выпилить try/except после выхода релиза 4.6.205
        try:
            result.update(check.getDynamicProperties())
        except:
            pass
        return result
