# -*- coding: utf-8 -*-
import sys
if sys.version_info.major == 3:
    import configparser
else:
    import ConfigParser as configparser

class ConfigProvider(object):
    def getBool(self, section, option, defaultValue):
        return defaultValue

    def getInt(self, section, option, defaultValue):
        return defaultValue

    def getString(self, section, option, defaultValue):
        return defaultValue

class ArtixConfigProvider(object):
    def __init__(self, artixConfig):
        self.config = artixConfig

    def __optName(self, section, option):
        return section + ':' + option

    def getBool(self, section, option, defaultValue):
        return self.config.getLogical(self.__optName(section, option), defaultValue)

    def getInt(self, section, option, defaultValue):
        return self.config.getInt(self.__optName(section, option), defaultValue)

    def getString(self, section, option, defaultValue):
        return self.config.getStringParameter(self.__optName(section, option), defaultValue)

class IniConfigProvider(object):
    def __init__(self, fileName):
        self.config = configparser.RawConfigParser()
        self.config.read(fileName)

    def getBool(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.getboolean(section, option)
        except:
            pass
        return value

    def getInt(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.getint(section, option)
        except:
            pass
        return value

    def getString(self, section, option, defaultValue):
        value = defaultValue
        try:
            value = self.config.get(section, option)
        except:
            pass
        return value

class Config(object):
    '''
    Настройки дисконта
    
    Может брать настройки из объекта ArtixConfig или ini файла
    Ini файл может потом пригодится для тестов
    loadFromArtixConfig - вызывается из кассы
    '''
    _instance = None
    def __new__(self, *args, **kwargs):
        if not self._instance:
            self._instance = super(Config, self).__new__(self, *args, **kwargs)
            self.data = ConfigProvider()
        return self._instance

    def reset(self):
        self.data = ConfigProvider()

    def loadFromArtixConfig(self, artixConfig):
        self.data = ArtixConfigProvider(artixConfig)

    def loadFromIni(self, fileName):
        self.data = IniConfigProvider(fileName)

    def getBool(self, section, option, defaultValue):
        return self.data.getBool(section, option, defaultValue)

    def getInt(self, section, option, defaultValue):
        return self.data.getInt(section, option, defaultValue)

    def getString(self, section, option, defaultValue):
        return self.data.getString(section, option, defaultValue)
