# -*- coding: utf-8 -*-

import yaml
import logging
import logging.config

from artixds.domain import DiscountSystem, DiscountCampaign
from subsystem import Subsystem
from dao import loadDiscountSystem
from dao import getDiscountUpdateTime
import dao

try:
    logging.config.fileConfig('/linuxcash/cash/discountsystems/logging.properties')
except Exception as e:
    print(u'logger configure error: `%s`' % e)
    logging.basicConfig()

session = dao.createSession()
discountSystem, secondDiscountSystem, bonusEarnSystem, bonusLimitSystem = DiscountSystem(), DiscountSystem(), DiscountSystem(), DiscountSystem()
time_update = loadDiscountSystem(session, discountSystem, secondDiscountSystem, bonusEarnSystem, bonusLimitSystem)

log = logging.getLogger("su.artix.dm")

if log.isEnabledFor(logging.DEBUG):
    log.debug(u'Первая дисконтная система:')
    for n, line in enumerate(filter(lambda x: x.rstrip(), yaml.dump(discountSystem).split('\n'))):
        log.debug(u'%4d: %s' % (n, line.rstrip()))
    log.debug(u'Вторая дисконтная система:')
    for n, line in enumerate(filter(lambda x: x.rstrip(), yaml.dump(secondDiscountSystem).split('\n'))):
        log.debug(u'%4d: %s' % (n, line.rstrip()))
    log.debug(u'Начисление бонусов:')
    for n, line in enumerate(filter(lambda x: x.rstrip(), yaml.dump(bonusEarnSystem).split('\n'))):
        log.debug(u'%4d: %s' % (n, line.rstrip()))
    log.debug(u'Списание бонусов:')
    for n, line in enumerate(filter(lambda x: x.rstrip(), yaml.dump(bonusLimitSystem).split('\n'))):
        log.debug(u'%4d: %s' % (n, line.rstrip()))

def refreshSystems():
    global time_update
    global manualCampaigns
    old_time_update = time_update
    log.debug(u'Обновление дисконтной системы')
    time_update = loadDiscountSystem(session, discountSystem, secondDiscountSystem, bonusEarnSystem, bonusLimitSystem, time_update)
    return old_time_update != time_update

def isUpToDate():
    global time_update
    global session
    updateTime, newSession = getDiscountUpdateTime(session)
    session = newSession
    return time_update is not None and updateTime == time_update

def calculateDiscountImpact(priority, checkPriority, qualifiers):
    subsystem = Subsystem(discountSystem)
    return calculateDiscountImpactForSubsystem(subsystem, priority, checkPriority, qualifiers)

def calculateSecondDiscountImpact(priority, checkPriority, qualifiers):
    subsystem = Subsystem(secondDiscountSystem)
    return calculateDiscountImpactForSubsystem(subsystem, priority, checkPriority, qualifiers)

def calculateDiscountImpactForSubsystem(subsystem, priority, checkPriority, qualifiers):
    filters = [lambda campaign: set(qualifiers) == (campaign.qualifiers)]
    subsystem.setQualifiersFilters(filters)
    return subsystem.getJsonImpacts(artix_core_check, priority, checkPriority)

def calculateBonusEarningsImpact():
    return Subsystem(bonusEarnSystem).getJsonImpacts(artix_core_check)

def calculateBonusLimitImpact():
    return Subsystem(bonusLimitSystem).getJsonImpacts(artix_core_check)

def getCampaigns():
    return Subsystem().encode(discountSystem.manualCampaigns)

def getSecondDiscountSystemCampaigns():
    return Subsystem().encode(secondDiscountSystem.manualCampaigns)

def getGroupAndCampaignsPriorities():
    return Subsystem().encode(discountSystem.groupsAndCampaignsPriorities)

def getSecondDiscountSystemGroupAndCampaignsPriorities():
    return Subsystem().encode(secondDiscountSystem.groupsAndCampaignsPriorities)

def calculateLifeTimeLimit(data):
    try:
        campaign = yaml.load(data)
        campaign.calcAssetDates()
        return {"assetBeginDate": campaign.assetBeginDate, "assetEndDate": campaign.assetEndDate, "id": campaign.id, "weight": campaign.weight}
    except Exception as e:
        log.warn('%s', str(e))
    return {}

